"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.optimizeStaticStyles = void 0;
const tslib_1 = require("tslib");
const t = tslib_1.__importStar(require("@babel/types"));
const core_1 = require("@taddy/core");
const getObjectPropertyKey_1 = require("./getObjectPropertyKey");
function optimizeStaticStyles(path) {
    const props = [];
    let quasis = [];
    let expressions = [];
    let currQuasi = '';
    for (const propPath of path.get('properties')) {
        if (!propPath.isObjectProperty()) {
            throw new Error('Cant optimize this path');
        }
        const keyPath = propPath.get('key');
        const key = (0, getObjectPropertyKey_1.getObjectPropertyKey)(keyPath);
        if (!key)
            continue;
        const valuePath = propPath.get('value');
        if (key === 'className' || key === 'style' || key === core_1.VARS_KEY) {
            if (currQuasi) {
                quasis.push(t.templateElement({ raw: currQuasi }));
                props.push(t.objectProperty(t.templateLiteral(quasis, expressions), t.booleanLiteral(true)));
            }
            props.push(propPath.node);
            quasis = [];
            expressions = [];
            currQuasi = '';
            continue;
        }
        // append " " delimiter for the continuous value or if that's not the first quasi
        if (currQuasi || quasis.length !== 0) {
            currQuasi += ' ';
        }
        if (valuePath.isStringLiteral()) {
            currQuasi += key + valuePath.node.value;
        }
        else if (valuePath.isBooleanLiteral()) {
            currQuasi += key;
        }
        else {
            currQuasi += key;
            quasis.push(t.templateElement({ raw: currQuasi }));
            expressions.push(valuePath.node);
            currQuasi = '';
        }
    }
    if (currQuasi) {
        quasis.push(t.templateElement({ raw: currQuasi }));
    }
    if (expressions.length === quasis.length) {
        quasis.push(t.templateElement({ raw: '' }));
    }
    if (quasis.length) {
        if (props.length === 0) {
            path.replaceWith(t.templateLiteral(quasis, expressions));
            return;
        }
        props.push(t.objectProperty(t.templateLiteral(quasis, expressions), t.booleanLiteral(true)));
    }
    path.node.properties = props;
}
exports.optimizeStaticStyles = optimizeStaticStyles;
//# sourceMappingURL=optimizeStaticStyles.js.map